using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Linq;
using KellermanSoftware.NetEncryptionLibrary;

namespace NetEncryptionLibraryDemo
{
    public partial class Demo : Form
    {
        // The 'target' is used to point to the textbox that the open/save
        //  dialog is currently associated with.
        private System.Windows.Forms.TextBox target;

        // One encryption object is used to perform all operations
        private Encryption encrypt;

        public Demo()
        {
            InitializeComponent();
            LoadEncryptionCombo();
            LoadSecureEraseCombo();
        }

        private void LoadSecureEraseCombo()
        {
            List<string> list = Enum.GetValues(typeof(SecureEraseAlgorithm))
                .Cast<SecureEraseAlgorithm>()
                .Select(v => v.ToString())
                .OrderBy(o => o)
                .ToList();


            cboSecureErase.Items.Clear();
            cboSecureErase.Items.AddRange(list.ToArray());
        }

        private void LoadEncryptionCombo()
        {
            cbxEncryptionType.Items.Clear();
            List<string> hashes = Enum.GetValues(typeof(HashProvider))
                .Cast<HashProvider>()
                .Select(v => v.ToString())
                .OrderBy(o => o)
                .ToList();

            List<string> crypt = Enum.GetValues(typeof(EncryptionProvider))
                .Cast<EncryptionProvider>()
                .Select(v => v.ToString())
                .Where(o => o != "RSA")
                .OrderBy(o => o)
                .ToList();

            var combined = new List<string>();
            combined.AddRange(crypt);
            combined.AddRange(hashes);

            cbxEncryptionType.Items.AddRange(combined.ToArray());
        }

        /// <summary>
        /// Show an error message box
        /// </summary>
        /// <param name="myException"></param>
        private void ShowError(Exception myException)
        {
            Hourglass(false);
            MessageBox.Show(myException.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            try
            {
                // Create the encryption object and start off with a randomly-generated
                //  initialization vector.
                encrypt = new Encryption();
                txtEncryptionIV.Text = encrypt.InitializationVector;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        private void openFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            // Update the associated text box
            target.Text = openFileDialog.FileName;
        }

        private void saveFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            
            // Update the associated text box
            target.Text = saveFileDialog.FileName;
        }

        private void btnBrowseCompressionInput_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtCompressionInput;
            openFileDialog.ShowDialog();
        }

        private void btnBrowseCompressionOutput_Click(object sender, EventArgs e)
        {
            // Show the save file dialog, associated with this text box
            target = txtCompressionOutput;
            saveFileDialog.ShowDialog();
        }

        private void btnBrowseFileErase_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtFileErase;
            openFileDialog.ShowDialog();
        }

        private void btnBrowseDirectoryErase_Click(object sender, EventArgs e)
        {
            // Save the results of the browse dialog into this text box
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
              txtDirectoryErase.Text = folderBrowserDialog.SelectedPath;
        }



        private void btnFileErase_Click(object sender, EventArgs e)
        {
            // Check to make sure the user selected a file
            if (txtFileErase.Text == "")
            {
                MessageBox.Show("Please select a file to erase");
                return;
            }

            try
            {
                Hourglass(true);
                // Securely erase the file
                SecureEraseAlgorithm algorithm = (SecureEraseAlgorithm) Enum.Parse(typeof(SecureEraseAlgorithm), cboSecureErase.Text);
                encrypt.SecureFileErase(txtFileErase.Text, algorithm);

                Hourglass(false);
                MessageBox.Show("Finished");
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnDirectoryErase_Click(object sender, EventArgs e)
        {
            // Check to make sure the user selected a directory
            if (txtDirectoryErase.Text == "")
            {
                MessageBox.Show("Please select a directory to erase");
                return;
            }

            try
            {
                Hourglass(true);

                // Securely erase the directory
                SecureEraseAlgorithm algorithm = (SecureEraseAlgorithm)Enum.Parse(typeof(SecureEraseAlgorithm), cboSecureErase.Text);
                encrypt.SecureDirectoryErase(txtDirectoryErase.Text,"*.*", SearchOption.AllDirectories, algorithm);

                Hourglass(false);
                MessageBox.Show("Finished");

            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }



        private void btnCompress_Click(object sender, EventArgs e)
        {
            try
            {
                // Check to make sure the user selected input and output files
                if (txtCompressionInput.Text == "")
                {
                    MessageBox.Show("Please select a file to compress");
                    return;
                }
                if (txtCompressionOutput.Text == "")
                {
                    MessageBox.Show("Please select a destination for the compressed file");
                    return;
                }

                Hourglass(true);

                // Do the actual compression
                switch (cbxCompressionType.Text)
                {
                    case "Deflate":
                        encrypt.CompressFile(CompressionType.Deflate, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                    case "GZip":
                        encrypt.CompressFile(CompressionType.GZip, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                }

                // Move the output file selection to the input file and clear the output file
                //  (so the user just has to choose an ouput file and then decompress)
                txtCompressionInput.Text = txtCompressionOutput.Text;
                txtCompressionOutput.Text = "";

                Hourglass(false);
                MessageBox.Show("Finished");
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnDecompress_Click(object sender, EventArgs e)
        {
            try
            {
                // Check to make sure the user selected input and output files
                if (txtCompressionInput.Text == "")
                {
                    MessageBox.Show("Please select a file to decompress");
                    return;
                }
                if (txtCompressionOutput.Text == "")
                {
                    MessageBox.Show("Please select a destination for the decompressed file");
                    return;
                }

                Hourglass(true);

                // Do the actual decompression
                switch (cbxCompressionType.Text)
                {
                    case "Deflate":
                        encrypt.DecompressFile(CompressionType.Deflate, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                    case "GZip":
                        encrypt.DecompressFile(CompressionType.GZip, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                }

                // Move the output file selection to the input file and clear the output file
                //  (so the user just has to choose an ouput file and then compress)
                txtCompressionInput.Text = txtCompressionOutput.Text;
                txtCompressionOutput.Text = "";

                Hourglass(false);
                MessageBox.Show("Finished");

            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnUUEncode_Click(object sender, EventArgs e)
        {
            try
            {
                // Do the uuEncoding, and display the result
                txtUUEncoded.Text = encrypt.uuEncode(txtUUPlaintext.Text);
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }

        }

        private void btnUUDecode_Click(object sender, EventArgs e)
        {
            try
            {
                // Do the uuDecoding, and display the result
                txtUUPlaintext.Text = encrypt.uuDecode(txtUUEncoded.Text);
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnEncrypt_Click(object sender, EventArgs e)
        {
            string fpeKey =
                "%87:0qd>u&-DiFXQQwD#gsTTz@2{=9b63(_j8^uRu]D`6Im#<^>Nyiy-+O;#5#+&cwB\"O%Q3&Q|>m}x\"HSUg6f*T6,i<.?2=1kcP^(~ORSc/Bb1`,&A\"ch<Z)*`&v)%cHjzz{hGKgR77g(N1y$RO{3 B\".z+AT2OelhEc}$5pN;0c{1gcUjY_7a8WTx.1xfLaCo@&@8nH(k7C~R~ 4]TIRY?Ld,me \\_\"g;Z9e}]=5RkuY<0xpO;]{bj6*+|B]c5V*.R24";


            try
            {
                // Grab the IV and salt parameters
                encrypt.InitializationVector = txtEncryptionIV.Text;
                encrypt.Salt = txtEncryptionSalt.Text;

                HashProvider hashType;
                EncryptionProvider cryptType;

                // Do the actual encryption and decryption, based on the algorithm specified
                if (Enum.TryParse<HashProvider>(cbxEncryptionType.Text, out hashType))
                {
                    txtEncryptionEncryptedString.Text = encrypt.HashStringHex(hashType, txtEncryptionString.Text);
                    txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                }
                else if (Enum.TryParse<EncryptionProvider>(cbxEncryptionType.Text, out cryptType))
                {
                    string key = txtEncryptionKey.Text;

                    //For demo purposes allow the user to enter a key that is too small for the Format Preserving Algorithm
                    if (cryptType == EncryptionProvider.FPEKELL1)
                    {
                        key += fpeKey.Substring(0, fpeKey.Length - key.Length);
                    }

                    txtEncryptionEncryptedString.Text = encrypt.EncryptString(cryptType, key, txtEncryptionString.Text);
                    txtEncryptionDecryptedString.Text = encrypt.DecryptString(cryptType, key, txtEncryptionEncryptedString.Text);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }

        }

        private void btnBrowseFileChecksum_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtFileChecksum;
            openFileDialog.ShowDialog();

            // Check to make sure the user selected a file
            if (txtFileChecksum.Text == "")
            {
                return;
            }

            try
            {
                txtCRC32.Text = encrypt.GetFileCRC32(txtFileChecksum.Text).ToString();
                txtAdler32.Text = encrypt.GetFileAdler32(txtFileChecksum.Text).ToString();
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        /// <summary>
        /// Turn on/off hourglass cursor
        /// </summary>
        /// <param name="enable"></param>
        public void Hourglass(bool enable)
        {
            if (enable)
            {
                Cursor.Current = Cursors.WaitCursor;
                Application.DoEvents();

            }
            else
            {
                Cursor.Current = Cursors.Default;
                Application.DoEvents();
            }
        }

        private void btnEncryptFile_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtFileToEncrypt;
            openFileDialog.ShowDialog();

            // Check to make sure the user selected a file
            if (txtFileToEncrypt.Text == "")
            {
                return;
            }

            try
            {
                string dir = Path.GetDirectoryName(txtFileToEncrypt.Text);
                string ext = Path.GetExtension(txtFileToEncrypt.Text);

                txtEncryptedFile.Text = Path.Combine(dir,"encrypted" + ext);
                txtDecryptedFile.Text = Path.Combine(dir,"decrypted" + ext);

                encrypt.EncryptFile(EncryptionProvider.Rijndael, "secret", txtFileToEncrypt.Text, txtEncryptedFile.Text);
                encrypt.DecryptFile(EncryptionProvider.Rijndael, "secret", txtEncryptedFile.Text, txtDecryptedFile.Text);
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnOneTimePassword_Click(object sender, EventArgs e)
        {
            txtOneTimePassword.Text =
                encrypt.CreateTimedOneTimePassword(txtOneTimePasswordKey.Text, (int) txtOneTimePasswordSeconds.Value);
        }
    }
}